@@ -1,6 +1,7 @@
 /*
  *  JIT detection for ClamAV bytecode.
  *
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2010 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -21,18 +22,17 @@
  */
 
 #include "llvm/ADT/Triple.h"
-#include "llvm/Support/raw_ostream.h"
-#ifdef LLVM29
-#include "llvm/Support/Host.h"
-#include "llvm/Support/DataTypes.h"
-#include "llvm/Support/Memory.h"
-#else
-#include "llvm/System/Host.h"
-#include "llvm/System/DataTypes.h"
-#include "llvm/System/Memory.h"
-#endif
-
 #include "llvm/Config/config.h"
+#include "llvm/Support/raw_ostream.h"
+#if LLVM_VERSION < 29
+#include "llvm/System/DataTypes.h"
+#include "llvm/System/Host.h"
+#include "llvm/System/Memory.h"
+#else
+#include "llvm/Support/DataTypes.h"
+#include "llvm/Support/Host.h"
+#include "llvm/Support/Memory.h"
+#endif
 
 extern "C" {
 #include "bytecode_detect.h"
@@ -45,7 +45,7 @@ static void warn_assumptions(const char *msg, int a, int b)
     errs() << "LibClamAV Warning: libclamav and llvm make inconsistent "
 	<< "assumptions about " << msg << ": " <<
 	a << " and " << b << "."
-	<< "Please report to http://bugs.clamav.net\n";
+	<< "Please report to https://bugzilla.clamav.net\n";
 }
 
 #define CASE_OS(theos, compat) case Triple::theos:\
@@ -56,16 +56,27 @@ static void warn_assumptions(const char *msg, int a, int b)
 
 void cli_detect_env_jit(struct cli_environment *env)
 {
+#if LLVM_VERSION < 31
     std::string host_triple = sys::getHostTriple();
+#else
+    std::string host_triple = sys::getDefaultTargetTriple();
+#endif
     INIT_STRFIELD(env->triple, host_triple.c_str());
 
     std::string cpu = sys::getHostCPUName();
     INIT_STRFIELD(env->cpu, cpu.c_str());
 
+#if LLVM_VERSION < 33
     if (env->big_endian != (int)sys::isBigEndianHost()) {
 	warn_assumptions("host endianness", env->big_endian, sys::isBigEndianHost());
 	env->big_endian = sys::isBigEndianHost();
     }
+#else
+    if (env->big_endian != (int)sys::IsBigEndianHost) {
+	warn_assumptions("host endianness", env->big_endian, sys::IsBigEndianHost);
+	env->big_endian = sys::IsBigEndianHost;
+    }
+#endif
 
 #ifdef __GNUC__
     env->cpp_version = MAKE_VERSION(0, __GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__);
@@ -136,20 +147,24 @@ void cli_detect_env_jit(struct cli_environment *env)
 	case Triple::UnknownOS:
 	    env->os = llvm_os_UnknownOS;
 	    break;
+#if LLVM_VERSION < 36
 	CASE_OS(AuroraUX, os_solaris);
 	CASE_OS(Cygwin, os_win32);
+	CASE_OS(MinGW32, os_win32);
+#endif
 	CASE_OS(Darwin, os_darwin);
 	CASE_OS(DragonFly, os_bsd);
 	CASE_OS(FreeBSD, os_bsd);
 	CASE_OS(Linux, os_linux);
 	CASE_OS(Lv2, os_unknown);
-	CASE_OS(MinGW32, os_win32);
-#ifndef LLVM29
+#if LLVM_VERSION < 29
 	CASE_OS(MinGW64, os_win64);
 #endif
 	CASE_OS(NetBSD,  os_bsd);
 	CASE_OS(OpenBSD, os_bsd);
+#if LLVM_VERSION < 31
 	CASE_OS(Psp, os_unknown);
+#endif
 	CASE_OS(Solaris, os_solaris);
 	case Triple::Win32:
 	     env->os = llvm_os_Win32;
