@@ -1,7 +1,12 @@
 /*
+ *  Copyright (C) 2015-2018 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
+ * 
+ *  Acknowledgements: The header structures were based upon "ELF: Executable 
+ *                    and Linkable Format, Portable Formats Specification, 
+ *                    Version 1.1".
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -114,7 +119,9 @@ static int cli_elf_fileheader(cli_ctx *ctx, fmap_t *map, union elf_file_hdr *fil
 	    break;
         default:
 	    cli_dbgmsg("ELF: Unknown ELF class (%u)\n", file_hdr->hdr64.e_ident[4]);
-	    return CL_EFORMAT;
+	    if (ctx)
+	      cli_append_virus(ctx, "Heuristics.Broken.Executable");
+	    return CL_VIRUS;
     }
 
     /* Need to know to endian convert */
@@ -367,11 +374,11 @@ static int cli_elf_ph64(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
 
             if(ctx) {
                 cli_dbgmsg("ELF: Segment #%d\n", i);
-                cli_dbgmsg("ELF: Segment type: 0x%x\n", EC32(program_hdr[i].p_type, conv));
-                cli_dbgmsg("ELF: Segment offset: 0x" STDx64 "\n", EC64(program_hdr[i].p_offset, conv));
-                cli_dbgmsg("ELF: Segment virtual address: 0x" STDx64 "\n", EC64(program_hdr[i].p_vaddr, conv));
-                cli_dbgmsg("ELF: Segment real size: 0x" STDx64 "\n", EC64(program_hdr[i].p_filesz, conv));
-                cli_dbgmsg("ELF: Segment virtual size: 0x" STDx64 "\n", EC64(program_hdr[i].p_memsz, conv));
+                cli_dbgmsg("ELF: Segment type: 0x" STDx32 "\n", (uint32_t) EC32(program_hdr[i].p_type, conv));
+                cli_dbgmsg("ELF: Segment offset: 0x" STDx64 "\n", (uint64_t) EC64(program_hdr[i].p_offset, conv));
+                cli_dbgmsg("ELF: Segment virtual address: 0x" STDx64 "\n", (uint64_t) EC64(program_hdr[i].p_vaddr, conv));
+                cli_dbgmsg("ELF: Segment real size: 0x" STDx64 "\n", (uint64_t) EC64(program_hdr[i].p_filesz, conv));
+                cli_dbgmsg("ELF: Segment virtual size: 0x" STDx64 "\n", (uint64_t) EC64(program_hdr[i].p_memsz, conv));
                 cli_dbgmsg("------------------------------------\n");
             }
         }
@@ -602,9 +609,9 @@ static int cli_elf_sh64(cli_ctx *ctx, fmap_t *map, struct cli_exe_info *elfinfo,
             elfinfo->section[i].rsz = EC64(section_hdr[i].sh_size, conv);
         }
         if(ctx) {
-	    cli_dbgmsg("ELF: Section %u\n", i);
-	    cli_dbgmsg("ELF: Section offset: " STDu64 "\n", EC64(section_hdr[i].sh_offset, conv));
-	    cli_dbgmsg("ELF: Section size: " STDu64 "\n", EC64(section_hdr[i].sh_size, conv));
+	    cli_dbgmsg("ELF: Section " STDu32 "\n", (uint32_t) i);
+	    cli_dbgmsg("ELF: Section offset: " STDu64 "\n", (uint64_t) EC64(section_hdr[i].sh_offset, conv));
+	    cli_dbgmsg("ELF: Section size: " STDu64 "\n", (uint64_t) EC64(section_hdr[i].sh_size, conv));
 
             sh_type = EC32(section_hdr[i].sh_type, conv);
             sh_flags = (uint32_t)(EC64(section_hdr[i].sh_flags, conv) & ELF_SHF_MASK);
@@ -815,11 +822,7 @@ int cli_scanelf(cli_ctx *ctx)
 int cli_elfheader(fmap_t *map, struct cli_exe_info *elfinfo)
 {
 	union elf_file_hdr file_hdr;
-	struct elf_section_hdr32 *section_hdr = NULL;
-	struct elf_program_hdr32 *program_hdr = NULL;
-	uint16_t shnum, phnum, shentsize, phentsize, i;
-	uint64_t entry, fentry = 0, shoff, phoff;
-	uint8_t conv = 0, err, is64 = 0;
+	uint8_t conv = 0, is64 = 0;
     int ret;
 
     cli_dbgmsg("in cli_elfheader\n");
