@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -80,12 +81,20 @@ typedef enum {
     CL_TYPE_JAVA,
     CL_TYPE_XAR,
     CL_TYPE_XZ,
+    CL_TYPE_OOXML_WORD,
+    CL_TYPE_OOXML_PPT,
+    CL_TYPE_OOXML_XL,
+    CL_TYPE_INTERNAL,
+    CL_TYPE_HWP3,
+    CL_TYPE_OOXML_HWP,
+    CL_TYPE_PS,
 
     /* Section for partition types */
     CL_TYPE_PART_ANY, /* unknown partition type */
     CL_TYPE_PART_HFSPLUS,
 
     /* bigger numbers have higher priority (in o-t-f detection) */
+    CL_TYPE_MBR,
     CL_TYPE_HTML, /* on the fly */
     CL_TYPE_MAIL,  /* magic + on the fly */
     CL_TYPE_SFX, /* foo SFX marker */
@@ -99,6 +108,16 @@ typedef enum {
     CL_TYPE_ISHIELD_MSI,
     CL_TYPE_ISO9660,
     CL_TYPE_DMG,
+    CL_TYPE_GPT,
+    CL_TYPE_APM,
+    CL_TYPE_XDP,
+    CL_TYPE_XML_WORD,
+    CL_TYPE_XML_XL,
+    CL_TYPE_XML_HWP,
+    CL_TYPE_HWPOLE2,
+    CL_TYPE_MHTML,
+
+    CL_TYPE_OTHER, /* on-the-fly, used for target 14 (OTHER) */
     CL_TYPE_IGNORED /* please don't add anything below */
 } cli_file_t;
 
