@@ -1,7 +1,7 @@
 /*
  *  Support for matcher using PCRE
  *
- *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2015, 2017 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *  All Rights Reserved.
  *
@@ -39,6 +39,13 @@
 #include "regex_pcre.h"
 
 #if HAVE_PCRE
+#if USING_PCRE2
+#define PCRE2_CODE_UNIT_WIDTH 8
+#include <pcre2.h>
+#else
+#include <pcre.h>
+#endif
+
 /* DEBUGGING */
 //#define MATCHER_PCRE_DEBUG
 #ifdef MATCHER_PCRE_DEBUG
@@ -172,7 +179,7 @@ void cli_pcre_perf_print()
     while (elem->run_count) {
         cli_infomsg (NULL, "%-*s %*lu %*lu %*llu %*.2f\n", max_name_len, elem->name,
                      8, elem->run_count, 8, elem->match_count,
-                     12, elem->usecs, 9, (double)elem->usecs/elem->run_count);
+                     12, (long long unsigned)elem->usecs, 9, (double)elem->usecs/elem->run_count);
         elem++;
     }
 }
@@ -337,6 +344,17 @@ int cli_pcre_addpatt(struct cli_matcher *root, const char *virname, const char *
             pm_dbgmsg("Matcher:  NONE\n");
 
         if (pm->pdata.options) {
+#if USING_PCRE2
+            pm_dbgmsg("Compiler: %s%s%s%s%s%s%s\n",
+                      pm->pdata.options & PCRE2_CASELESS ? "PCRE2_CASELESS " : "",
+                      pm->pdata.options & PCRE2_DOTALL ? "PCRE2_DOTALL " : "",
+                      pm->pdata.options & PCRE2_MULTILINE ? "PCRE2_MULTILINE " : "",
+                      pm->pdata.options & PCRE2_EXTENDED ? "PCRE2_EXTENDED " : "",
+
+                      pm->pdata.options & PCRE2_ANCHORED ? "PCRE2_ANCHORED " : "",
+                      pm->pdata.options & PCRE2_DOLLAR_ENDONLY ? "PCRE2_DOLLAR_ENDONLY " : "",
+                      pm->pdata.options & PCRE2_UNGREEDY ? "PCRE2_UNGREEDY " : "");
+#else
             pm_dbgmsg("Compiler: %s%s%s%s%s%s%s\n",
                       pm->pdata.options & PCRE_CASELESS ? "PCRE_CASELESS " : "",
                       pm->pdata.options & PCRE_DOTALL ? "PCRE_DOTALL " : "",
@@ -346,6 +364,7 @@ int cli_pcre_addpatt(struct cli_matcher *root, const char *virname, const char *
                       pm->pdata.options & PCRE_ANCHORED ? "PCRE_ANCHORED " : "",
                       pm->pdata.options & PCRE_DOLLAR_ENDONLY ? "PCRE_DOLLAR_ENDONLY " : "",
                       pm->pdata.options & PCRE_UNGREEDY ? "PCRE_UNGREEDY " : "");
+#endif
         }
         else
             pm_dbgmsg("Compiler: NONE\n");
@@ -409,11 +428,15 @@ int cli_pcre_build(struct cli_matcher *root, long long unsigned match_limit, lon
         }
 
         /* options override through metadata manipulation */
-#ifdef PCRE_NEVER_UTF
-        pm->pdata.options |= PCRE_NEVER_UTF; /* implemented in 8.33, disables (?UTF*) potential security vuln */
-#endif
+#if USING_PCRE2
+        //pm->pdata.options |= PCRE2_NEVER_UTF; /* disables (?UTF*) potential security vuln */
+        //pm->pdata.options |= PCRE2_UCP;
+        //pm->pdata.options |= PCRE2_AUTO_CALLOUT; /* used with CALLOUT(-BACK) function */
+#else
+        //pm->pdata.options |= PCRE_NEVER_UTF; /* implemented in 8.33, disables (?UTF*) potential security vuln */
         //pm->pdata.options |= PCRE_UCP;/* implemented in 8.20 */
         //pm->pdata.options |= PCRE_AUTO_CALLOUT; /* used with CALLOUT(-BACK) function */
+#endif
 
         if (dconf && (dconf->pcre & PCRE_CONF_OPTIONS)) {
             /* compile the regex, no options override *wink* */
@@ -421,7 +444,7 @@ int cli_pcre_build(struct cli_matcher *root, long long unsigned match_limit, lon
             ret = cli_pcre_compile(&(pm->pdata), match_limit, recmatch_limit, 0, 0);
         }
         else {
-            /* compile the regex, options overrided and disabled */
+            /* compile the regex, options overridden and disabled */
             pm_dbgmsg("cli_pcre_build: Compiling regex: /%s/ (without options)\n", pm->pdata.expression);
             ret = cli_pcre_compile(&(pm->pdata), match_limit, recmatch_limit, 0, 1);
         }
@@ -564,19 +587,19 @@ int cli_pcre_scanbuf(const unsigned char *buffer, uint32_t length, const char **
 {
     struct cli_pcre_meta **metatable = root->pcre_metatable, *pm = NULL;
     struct cli_pcre_data *pd;
+    struct cli_pcre_results p_res;
     struct cli_ac_result *newres;
     uint32_t adjbuffer, adjshift, adjlength;
     unsigned int i, evalcnt = 0;
     uint64_t maxfilesize, evalids = 0;
     uint32_t global, encompass, rolling;
-    int rc, lrc, offset, options=0, ovector[OVECCOUNT];
+    int rc = 0, offset = 0, ret = CL_SUCCESS, options=0;
     uint8_t viruses_found = 0;
 
-    if ((!root->pcre_metatable) || (ctx && ctx->dconf && !(ctx->dconf->pcre & PCRE_CONF_SUPPORT))) {
+    if ((root->pcre_metas == 0) || (!root->pcre_metatable) || (ctx && ctx->dconf && !(ctx->dconf->pcre & PCRE_CONF_SUPPORT)))
         return CL_SUCCESS;
-    }
 
-    /* NOTE: moved pcre maxfilesize limit check to caller [matcher_run] */
+    memset(&p_res, 0, sizeof(p_res));
 
     for (i = 0; i < root->pcre_metas; ++i) {
         pm = root->pcre_metatable[i];
@@ -596,7 +619,7 @@ int cli_pcre_scanbuf(const unsigned char *buffer, uint32_t length, const char **
 
         /* evaluate trigger */
         if (pm->lsigid[0]) {
-            cli_dbgmsg("cli_pcre_scanbuf: checking %s; running regex /%s/\n", pm->trigger, pd->expression);
+            pm_dbgmsg("cli_pcre_scanbuf: checking %s; running regex /%s/\n", pm->trigger, pd->expression);
 #ifdef PCRE_BYPASS
             if (strcmp(pm->trigger, PCRE_BYPASS))
 #endif
@@ -604,7 +627,7 @@ int cli_pcre_scanbuf(const unsigned char *buffer, uint32_t length, const char **
                     continue;
         }
         else {
-            cli_dbgmsg("cli_pcre_scanbuf: skipping %s check due to unintialized lsigid\n", pm->trigger);
+            cli_dbgmsg("cli_pcre_scanbuf: skipping %s check due to uninitialized lsigid\n", pm->trigger);
             /* fall-through to unconditional execution - sigtool-only */
         }
 
@@ -613,7 +636,7 @@ int cli_pcre_scanbuf(const unsigned char *buffer, uint32_t length, const char **
         rolling = (pm->flags & CLI_PCRE_ROLLING);     /* rolling search (unanchored) */
         offset = pd->search_offset;                   /* this is usually 0 */
 
-        cli_dbgmsg("cli_pcre_scanbuf: triggered %s; running regex /%s/%s%s\n", pm->trigger, pd->expression, 
+        pm_dbgmsg("cli_pcre_scanbuf: triggered %s; running regex /%s/%s%s\n", pm->trigger, pd->expression, 
                    global ? " (global)":"", rolling ? " (rolling)":"");
 
         /* adjust the buffer sent to cli_pcre_match for offset and maxshift */
@@ -628,7 +651,11 @@ int cli_pcre_scanbuf(const unsigned char *buffer, uint32_t length, const char **
 
         /* check for need to anchoring */
         if (!rolling && !adjshift && (adjbuffer != CLI_OFF_ANY))
+#if USING_PCRE2
+            options |= PCRE2_ANCHORED;
+#else
             options |= PCRE_ANCHORED;
+#endif
         else
             options = 0;
 
@@ -660,23 +687,27 @@ int cli_pcre_scanbuf(const unsigned char *buffer, uint32_t length, const char **
 
         /* if the global flag is set, loop through the scanning */
         do {
+            /* reset the match results */
+            if ((ret = cli_pcre_results_reset(&p_res, pd)) != CL_SUCCESS)
+                break;
+
             /* performance metrics */
             cli_event_time_start(p_sigevents, pm->sigtime_id);
-            rc = cli_pcre_match(pd, buffer+adjbuffer, adjlength, offset, options, ovector, OVECCOUNT);
+            rc = cli_pcre_match(pd, buffer+adjbuffer, adjlength, offset, options, &p_res);
             cli_event_time_stop(p_sigevents, pm->sigtime_id);
             /* if debug, generate a match report */
             if (cli_debug_flag)
-                cli_pcre_report(pd, buffer+adjbuffer, adjlength, rc, ovector, OVECCOUNT);
+                cli_pcre_report(pd, buffer+adjbuffer, adjlength, rc, &p_res);
 
             /* matched, rc shouldn't be >0 unless a full match occurs */
             if (rc > 0) {
-                cli_dbgmsg("cli_pcre_scanbuf: located regex match @ %d\n", adjbuffer+ovector[0]);
+                cli_dbgmsg("cli_pcre_scanbuf: located regex match @ %d\n", adjbuffer+p_res.match[0]);
 
                 /* check if we've gone over offset+shift */
                 if (!encompass && adjshift) {
-                    if (ovector[0] > adjshift) {
+                    if (p_res.match[0] > adjshift) {
                         /* ignore matched offset (outside of maxshift) */
-                        cli_dbgmsg("cli_pcre_scanbuf: match found outside of maxshift @%u\n", adjbuffer+ovector[0]);
+                        cli_dbgmsg("cli_pcre_scanbuf: match found outside of maxshift @%u\n", adjbuffer+p_res.match[0]);
                         break;
                     }
                 }
@@ -687,69 +718,59 @@ int cli_pcre_scanbuf(const unsigned char *buffer, uint32_t length, const char **
                 /* for logical signature evaluation */
                 if (pm->lsigid[0]) {
                     pm_dbgmsg("cli_pcre_scanbuf: assigning lsigcnt[%d][%d], located @ %d\n",
-                              pm->lsigid[1], pm->lsigid[2], adjbuffer+ovector[0]);
+                              pm->lsigid[1], pm->lsigid[2], adjbuffer+p_res.match[0]);
 
-                    lrc = lsig_sub_matched(root, mdata, pm->lsigid[1], pm->lsigid[2], adjbuffer+ovector[0], 0);
-                    if (lrc != CL_SUCCESS)
-                        return lrc;
+                    ret = lsig_sub_matched(root, mdata, pm->lsigid[1], pm->lsigid[2], adjbuffer+p_res.match[0], 0);
+                    if (ret != CL_SUCCESS)
+                        break;
                 } else {
                     /* for raw match data - sigtool only */
                     if(res) {
                         newres = (struct cli_ac_result *)cli_calloc(1, sizeof(struct cli_ac_result));
                         if(!newres) {
                             cli_errmsg("cli_pcre_scanbuff: Can't allocate memory for new result\n");
-                            return CL_EMEM;
+                            ret = CL_EMEM;
+                            break;
                         }
                         newres->virname = pm->virname;
                         newres->customdata = NULL; /* get value? */
                         newres->next = *res;
-                        newres->offset = adjbuffer+ovector[0];
+                        newres->offset = adjbuffer+p_res.match[0];
                         *res = newres;
                     } else {
-                        if (ctx && SCAN_ALL) {
-                            viruses_found = 1;
-                            cli_append_virus(ctx, (const char *)pm->virname);
-                        }
+                        ret = CL_CLEAN;
+                        viruses_found = 1;
+                        if (ctx)
+                            ret = cli_append_virus(ctx, (const char *)pm->virname);
                         if (virname)
                             *virname = pm->virname;
                         if (!ctx || !SCAN_ALL)
-                            return CL_VIRUS;
+                            if (ret != CL_CLEAN)
+                                break;
                     }
                 }
             }
 
             /* move off to the end of the match for next match; offset is relative to adjbuffer
              * NOTE: misses matches starting within the last match; TODO: start from start of last match? */
-            offset = ovector[1];
-
-            /* clear the ovector results (they fall through the pcre_match) */
-            memset(ovector, 0, sizeof(ovector));
+            offset = p_res.match[1];
         } while (global && rc > 0 && offset < adjlength);
 
-        /* handle error codes */
-        if (rc < 0 && rc != PCRE_ERROR_NOMATCH) {
-            switch (rc) {
-            case PCRE_ERROR_CALLOUT:
-                break;
-            case PCRE_ERROR_NOMEMORY:
-                cli_errmsg("cli_pcre_scanbuf: cli_pcre_match: pcre_exec: out of memory\n");
-                return CL_EMEM;
-            case PCRE_ERROR_MATCHLIMIT:
-                cli_dbgmsg("cli_pcre_scanbuf: cli_pcre_match: pcre_exec: match limit exceeded\n");
-                break;
-            case PCRE_ERROR_RECURSIONLIMIT:
-                cli_dbgmsg("cli_pcre_scanbuf: cli_pcre_match: pcre_exec: recursive limit exceeded\n");
-                break;
-            default:
-                cli_errmsg("cli_pcre_scanbuf: cli_pcre_match: pcre_exec: returned error %d\n", rc);
-                return CL_BREAK;
-            }
-        }
+        /* handle error code */
+        if (rc < 0 && p_res.err != CL_SUCCESS)
+            ret = p_res.err;
+
+        /* jumps out of main loop from 'global' loop */
+        if (ret != CL_SUCCESS)
+            break;
     }
 
-    if (viruses_found)
+    /* free match results */
+    cli_pcre_results_free(&p_res);
+
+    if (ret == CL_SUCCESS && viruses_found)
         return CL_VIRUS;
-    return CL_SUCCESS;
+    return ret;
 }
 
 void cli_pcre_freemeta(struct cli_matcher *root, struct cli_pcre_meta *pm)
