@@ -5,6 +5,7 @@
  *                          Greg Roelofs <newt@pobox.com>,
  *                          John Bowler <jbowler@acm.org>,
  *                          Tom Lane <tgl@sss.pgh.pa.us>
+ *   Copyright (C) 2015, 2017 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *   Copyright (C) 2011 Sourcefire, Inc.
  *   Maintainer: Tomasz Kojm <tkojm@clamav.net>
  *
@@ -56,11 +57,6 @@ typedef unsigned long  ulg;
 #define CRITICAL(chunkID)   (!ANCILLARY(chunkID))
 #define PUBLIC(chunkID)     (!PRIVATE(chunkID))
 
-/* what the PNG, MNG and JNG magic numbers should be */
-static const uch good_PNG_magic[8] = {137, 80, 78, 71, 13, 10, 26, 10};
-static const uch good_MNG_magic[8] = {138, 77, 78, 71, 13, 10, 26, 10};
-static const uch good_JNG_magic[8] = {139, 74, 78, 71, 13, 10, 26, 10};
-
 /* GRR FIXME:  could merge all three of these into single table (bit fields) */
 
 /* GRR 20061203:  for "isalpha()" that works even on EBCDIC machines */
@@ -714,7 +710,7 @@ int cli_parsepng(cli_ctx *ctx)
             } else if (cur_y >= h) {
                 inflateEnd(&zstrm);
 		if(eod - p > 0) {
-		    cli_dbgmsg("PNG:  %li bytes remaining in buffer before inflateEnd()", eod-p);
+		    cli_dbgmsg("PNG:  %u bytes remaining in buffer before inflateEnd()", (unsigned int)(eod - p));
 		    return CL_EPARSE;
 		}
 		err = Z_STREAM_END;
