@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009 Sourcefire, Inc.
  *
  *  Authors: aCaB <acab@clamav.net>
@@ -21,6 +22,10 @@
 #ifndef __SNPRINTF_H
 #define __SNPRINTF_H
 
+#ifndef HAVE_SNPRINTF
+
 int snprintf(char *str, size_t size, const char *format, ...);
 
+#endif
+
 #endif /* __SNPRINTF_H */
\ No newline at end of file
